--===========
-- minGUI
--
-- by DjPoke
-- (c) 2023
--===========

-- function to call from love.load()
function init_minGUI()
	-- set default filter
	love.graphics.setDefaultFilter("nearest", "nearest")
	
	-- constants
	MG_PANEL = 1
	MG_BUTTON = 2
	
	MG_PANEL_IMAGE = 1
	MG_BUTTON_UP_IMAGE = 2
	MG_BUTTON_DOWN_IMAGE = 3

	-- init minGUI table
	minGUI = {
		-- attributs
		bgcolor = {r = 0.5, g = 0.5, b = 0.5, a = 1.0},
		ptree = {},
		gtree = {},
		sprite = {},
		-- methodes
		set_bgcolor = function(self, r, g, b, a)
			minGUI.bgcolor.r = r
			minGUI.bgcolor.g = g
			minGUI.bgcolor.b = b
			minGUI.bgcolor.a = a
		end,
		add_panel = function(self, num, x, y, width, height)
			minGUI.ptree[num] = {
				num = num, tp = MG_PANEL, x = x, y = y, width = width, height = height,
				canvas = love.graphics.newCanvas(width, height)
			}
		end,
		add_button = function(self, num, x, y, width, height, text, parent)
			if parent == nil or minGUI.ptree[num] ~= nil then
				minGUI.gtree[num] = {
					num = num, tp = MG_BUTTON, x = x, y = y, width = width, height = height, text = text, parent = parent, down = false,
					canvas = love.graphics.newCanvas(width, height)
				}
			end
		end
	}
	
	-- load 9-slice sprites
	load_9slice(MG_PANEL_IMAGE, "minGUI/panel.png")
	load_9slice(MG_BUTTON_UP_IMAGE, "minGUI/button_up.png")
	load_9slice(MG_BUTTON_DOWN_IMAGE, "minGUI/button_down.png")
end

-- minGUI event mouse pressed, to call from love.mousepressed()
function mouse_pressed_minGUI(x, y, button)
	-- todo
end

-- minGUI event mouse released, to call from love.mousereleased()
function mouse_released_minGUI(x, y, button)
	-- todo
end

-- minGUI event key pressed, to call from love.keypressed()
function key_pressed_minGUI(key)
	-- todo
end

-- minGUI event key released, to call from love.keyreleased()
function key_released_minGUI(key)
	-- todo
end

-- function to call from love.draw()
function draw_minGUI()
	love.graphics.clear(minGUI.bgcolor.r, minGUI.bgcolor.g, minGUI.bgcolor.b, minGUI.bgcolor.a)

	-- draw pannels and their gadgets
	for i, v in ipairs(minGUI.ptree) do
		-- draw panels first
		if v.tp == MG_PANEL then
			draw_9slice(MG_PANEL_IMAGE, v.width, v.height, v.canvas)
			
			love.graphics.draw(v.canvas, v.x, v.y)

			-- draw sons gadgets
			for j, w in ipairs(minGUI.gtree) do
				if w.tp == MG_BUTTON and w.parent == v.num then
					if not w.down then
						draw_9slice(MG_BUTTON_UP_IMAGE, w.width, w.height, w.canvas)
					else
						draw_9slice(MG_BUTTON_DOWN_IMAGE, w.width, w.height, w.canvas)
					end
					
					love.graphics.draw(w.canvas, v.x + w.x, v.y + w.y)
				end
			end
		end
	end
	
	-- draw gadgets without pannels
	for i, v in ipairs(minGUI.gtree) do
		if v.tp == MG_BUTTON and v.parent == nil then
			if not v.down then
				draw_9slice(MG_BUTTON_UP_IMAGE, v.x, v.y, v.width, v.height)
			else
				draw_9slice(MG_BUTTON_DOWN_IMAGE, v.x, v.y, v.width, v.height)
			end
		end
	end
end

-- load 9-slice sprite
function load_9slice(num, filename)
	minGUI.sprite[num] = love.graphics.newImage(filename)
end

-- draw a 9-slice sprite
function draw_9slice(num, width, height, canvas)
	-- can't go under a special size
	if width <= 0 then return end
	if height <= 0 then return end
	
	-- parts of images
	local p = {}
	
	-- quads size x & y
	local qsx = minGUI.sprite[num]:getWidth() / 3
	local qsy = minGUI.sprite[num]:getHeight() / 3
	
	-- maximum number of rows and columns of quads
	local tx = math.floor(width / qsx)
	local ty = math.floor(height / qsy)

	-- unused pieces of tiles
	local px = (width - (tx * qsx)) / tx
	local py = (height - (ty * qsy)) / ty

	-- real size of tiles
	local rsx = qsx + px
	local rsy = qsy + py
	
	-- scale x & y values
	local sx = rsx / qsx
	local sy = rsy / qsy
	
	-- get limits
	local limitx = tx - 1
	local limity = ty - 1
	
	-- separate texture in 9 quads
	p[1] = love.graphics.newQuad(0, 0, qsx, qsy, minGUI.sprite[num])
	p[2] = love.graphics.newQuad(qsx, 0, qsx, qsy, minGUI.sprite[num])
	p[3] = love.graphics.newQuad(qsx * 2, 0, qsx, qsy, minGUI.sprite[num])
	p[4] = love.graphics.newQuad(0, qsy, qsx, qsy, minGUI.sprite[num])
	p[5] = love.graphics.newQuad(qsx, qsy, qsx, qsy, minGUI.sprite[num])
	p[6] = love.graphics.newQuad(qsx * 2, qsy, qsx, qsy, minGUI.sprite[num])
	p[7] = love.graphics.newQuad(0, qsy * 2, qsx, qsy, minGUI.sprite[num])
	p[8] = love.graphics.newQuad(qsx, qsy * 2, qsx, qsy, minGUI.sprite[num])
	p[9] = love.graphics.newQuad(qsx * 2, qsy * 2, qsx, qsy, minGUI.sprite[num])

	-- draw the quads on the canvas
	--=============================

	-- render to the canvas
	love.graphics.setCanvas(canvas)

	-- draw top left corner
	love.graphics.draw(minGUI.sprite[num], p[1], 0, 0, 0, sx, sy)

	-- draw top right corner
	love.graphics.draw(minGUI.sprite[num], p[3], (limitx * rsx), 0, 0, sx, sy)

	-- draw bottom left corner
	love.graphics.draw(minGUI.sprite[num], p[7], 0, (limity * rsy), 0, sx, sy)
			
	-- draw bottom right corner
	love.graphics.draw(minGUI.sprite[num], p[9], (limitx * rsx), (limity * rsy), 0, sx, sy)

	-- draw edges
	for x9 = 1, limitx - 1 do
		love.graphics.draw(minGUI.sprite[num], p[2], (x9 * rsx), 0, 0, sx, sy)
		love.graphics.draw(minGUI.sprite[num], p[8], (x9 * rsx), (limity * rsy), 0, sx, sy)
	end
	
	for y9 = 1, limity - 1 do
		love.graphics.draw(minGUI.sprite[num], p[4], 0, (y9 * rsy), 0, sx, sy)
		love.graphics.draw(minGUI.sprite[num], p[6], (limitx * rsx), (y9 * rsy), 0, sx, sy)
	end
		
	-- draw center
	for y9 = 1, limity - 1 do
		for x9 = 1, limitx - 1 do		
			love.graphics.draw(minGUI.sprite[num], p[5], (x9 * rsx), (y9 * rsy), 0, sx, sy)
		end
	end
	
	-- render to window
	love.graphics.setCanvas()
end